from aqsc.math_utilities import *
lambda_B_psi_nm2_in_X_n = lambda n_eval, X_coef_cp, Y_coef_cp, B_theta_coef_cp, B_psi_coef_cp, iota_coef, p_perp_coef_cp, Delta_coef_cp, B_denom_coef_c, B_alpha_coef, kap_p, dl_p, tau_p: \
    -(diff(X_coef_cp[1],True,1)*diff(Y_coef_cp[1],False,1)-iota_coef[0]*X_coef_cp[1]*diff(Y_coef_cp[1],True,2)-X_coef_cp[1]*diff(Y_coef_cp[1],True,1,False,1)-diff(X_coef_cp[1],False,1)*diff(Y_coef_cp[1],True,1)+iota_coef[0]*Y_coef_cp[1]*diff(X_coef_cp[1],True,2)+Y_coef_cp[1]*diff(X_coef_cp[1],True,1,False,1))/(dl_p*kap_p*n_eval-dl_p*kap_p)
lambda_dphi_B_psi_nm2_in_X_n = lambda n_eval, X_coef_cp, Y_coef_cp, B_theta_coef_cp, B_psi_coef_cp, iota_coef, p_perp_coef_cp, Delta_coef_cp, B_denom_coef_c, B_alpha_coef, kap_p, dl_p, tau_p: \
    (X_coef_cp[1]*diff(Y_coef_cp[1],True,1)-Y_coef_cp[1]*diff(X_coef_cp[1],True,1))/(dl_p*kap_p*n_eval-dl_p*kap_p)
lambda_dchi_B_psi_nm2_in_X_n = lambda n_eval, X_coef_cp, Y_coef_cp, B_theta_coef_cp, B_psi_coef_cp, iota_coef, p_perp_coef_cp, Delta_coef_cp, B_denom_coef_c, B_alpha_coef, kap_p, dl_p, tau_p: \
    (iota_coef[0]*X_coef_cp[1]*diff(Y_coef_cp[1],True,1)-iota_coef[0]*Y_coef_cp[1]*diff(X_coef_cp[1],True,1))/(dl_p*kap_p*n_eval-dl_p*kap_p)
lambda_B_psi_nm2_in_Z_n = lambda n_eval, X_coef_cp, Y_coef_cp, B_theta_coef_cp, B_psi_coef_cp, iota_coef, p_perp_coef_cp, Delta_coef_cp, B_denom_coef_c, B_alpha_coef, kap_p, dl_p, tau_p: \
    (X_coef_cp[1]*diff(Y_coef_cp[1],True,1)-Y_coef_cp[1]*diff(X_coef_cp[1],True,1))/(n_eval-1)
lambda_B_psi_nm2_in_p_n = lambda n_eval, X_coef_cp, Y_coef_cp, B_theta_coef_cp, B_psi_coef_cp, iota_coef, p_perp_coef_cp, Delta_coef_cp, B_denom_coef_c, B_alpha_coef, kap_p, dl_p, tau_p: \
    -(2*diff(Delta_coef_cp[0],False,1))/(B_alpha_coef[0]*B_denom_coef_c[0]*n_eval-B_alpha_coef[0]*B_denom_coef_c[0])
lambda_dphi_B_psi_nm2_in_p_n = lambda n_eval, X_coef_cp, Y_coef_cp, B_theta_coef_cp, B_psi_coef_cp, iota_coef, p_perp_coef_cp, Delta_coef_cp, B_denom_coef_c, B_alpha_coef, kap_p, dl_p, tau_p: \
    -(2*Delta_coef_cp[0]-2)/(B_alpha_coef[0]*B_denom_coef_c[0]*n_eval-B_alpha_coef[0]*B_denom_coef_c[0])
lambda_dchi_B_psi_nm2_in_p_n = lambda n_eval, X_coef_cp, Y_coef_cp, B_theta_coef_cp, B_psi_coef_cp, iota_coef, p_perp_coef_cp, Delta_coef_cp, B_denom_coef_c, B_alpha_coef, kap_p, dl_p, tau_p: \
    -((2*Delta_coef_cp[0]-2)*iota_coef[0])/(B_alpha_coef[0]*B_denom_coef_c[0]*n_eval-B_alpha_coef[0]*B_denom_coef_c[0])
lambda_B_psi_nm2_in_Delta_n = lambda n_eval, X_coef_cp, Y_coef_cp, B_theta_coef_cp, B_psi_coef_cp, iota_coef, p_perp_coef_cp, Delta_coef_cp, B_denom_coef_c, B_alpha_coef, kap_p, dl_p, tau_p: \
    (2*diff(Delta_coef_cp[0],False,1))/(B_alpha_coef[0]*n_eval-B_alpha_coef[0])
lambda_dphi_B_psi_nm2_in_Delta_n = lambda n_eval, X_coef_cp, Y_coef_cp, B_theta_coef_cp, B_psi_coef_cp, iota_coef, p_perp_coef_cp, Delta_coef_cp, B_denom_coef_c, B_alpha_coef, kap_p, dl_p, tau_p: \
    (2*Delta_coef_cp[0]-2)/(B_alpha_coef[0]*n_eval-B_alpha_coef[0])
lambda_dchi_B_psi_nm2_in_Delta_n = lambda n_eval, X_coef_cp, Y_coef_cp, B_theta_coef_cp, B_psi_coef_cp, iota_coef, p_perp_coef_cp, Delta_coef_cp, B_denom_coef_c, B_alpha_coef, kap_p, dl_p, tau_p: \
    ((2*Delta_coef_cp[0]-2)*iota_coef[0])/(B_alpha_coef[0]*n_eval-B_alpha_coef[0])



from aqsc.math_utilities import *

def eval_B_theta_coefs(p_perp_coef_cp, Delta_coef_cp, B_denom_coef_c, iota_coef):
    coef_B_theta = -(
        2*B_denom_coef_c[0]**2*(diff(p_perp_coef_cp[1],False,1))
        +2*B_denom_coef_c[0]**2*iota_coef[0]*(diff(p_perp_coef_cp[1],True,1))
        +(Delta_coef_cp[0]-2)*iota_coef[0]*(diff(B_denom_coef_c[1],True,1))
        +4*B_denom_coef_c[0]*B_denom_coef_c[1]*(diff(p_perp_coef_cp[0],False,1))
        +2*B_denom_coef_c[1]*(diff(Delta_coef_cp[0],False,1)))/(2)
    coef_dchi_B_theta = B_denom_coef_c[0]*iota_coef[0]*Delta_coef_cp[1]
    coef_dphi_B_theta = B_denom_coef_c[0]*Delta_coef_cp[1]

    return({
        'coef_B_theta':coef_B_theta,
        'coef_dchi_B_theta':coef_dchi_B_theta,
        'coef_dphi_B_theta':coef_dphi_B_theta,
    })



import unittest
import numpy as np
from aqsc import *

import jax.numpy as jnp

# The phi derivative tests are done with splines, for which the 
# specral method is not very well-suited.
diff_tolerance = 5e-4

# Size of the chi and phi grid used for evaluation tests
points = np.linspace(0, 2*np.pi*(1-1/n_grid_phi), n_grid_phi)
chi = np.linspace(0, 2*np.pi*(1-1/n_grid_chi), n_grid_chi)
phi = points
psi = np.linspace(0,5,100)

def is_roughly_close(array_a, array_b):
    return(jnp.abs(array_a-array_b)<diff_tolerance)

class TestCalculus(unittest.TestCase):

    def test_dchi_ichi(self):
        ''' 
        chi derivatives and antiderivatives 
        '''
        # max chi mode number is even
        nfp = np.random.randint(4)+2
        rands_i = np.random.randint(1,5, size=12)
        amp=0.2
        test1 = amp*ChiPhiFunc(np.array([
                np.sin(rands_i[0]*points)+np.cos(rands_i[1]*points), 
                np.sin(rands_i[0]*points)+np.cos(rands_i[1]*points),
                np.sin(rands_i[2]*points)+np.cos(rands_i[3]*points)
        ]), nfp, trig_mode = True)

        test1_int = amp*ChiPhiFunc(np.array([
                np.sin(rands_i[0]*points)+np.cos(rands_i[1]*points), 
                np.zeros_like(points, dtype=np.complex128),
                np.sin(rands_i[2]*points)+np.cos(rands_i[3]*points)
        ]), nfp, trig_mode = True)

        dchi_test1 = amp*ChiPhiFunc(np.array([
                -2*(np.sin(rands_i[2]*points)+np.cos(rands_i[3]*points)), 
                np.zeros_like(points, dtype=np.complex128),
                2*(np.sin(rands_i[0]*points)+np.cos(rands_i[1]*points))
        ]), nfp, trig_mode = True)

        guess_dchi = test1.dchi()
        guess_ichi = dchi_test1.antid_chi()
        # chi derivative
        self.assertTrue(jnp.all(jnp.isclose(guess_dchi.content, dchi_test1.content)))
        # chi integral
        self.assertTrue(jnp.all(jnp.isclose(guess_ichi.content, test1_int.content)))
        
        # max chi mode number is odd
        nfp = np.random.randint(4)+2
        rands_i = np.random.randint(1,5, size=12)

        test1 = ChiPhiFunc(np.array([
                np.sin(rands_i[0]*points)+np.cos(rands_i[1]*points), 
                np.sin(rands_i[2]*points)+np.cos(rands_i[3]*points), 
                np.sin(rands_i[4]*points)+np.cos(rands_i[5]*points), 
                np.sin(rands_i[6]*points)+np.cos(rands_i[7]*points)
        ]), nfp, trig_mode = True)

        dchi_test1 = ChiPhiFunc(np.array([
                -3*(np.sin(rands_i[6]*points)+np.cos(rands_i[7]*points)), 
                -(np.sin(rands_i[4]*points)+np.cos(rands_i[5]*points)), 
                np.sin(rands_i[2]*points)+np.cos(rands_i[3]*points), 
                3*(np.sin(rands_i[0]*points)+np.cos(rands_i[1]*points))
        ]), nfp, trig_mode = True)

        guess_dchi = test1.dchi()
        guess_ichi = dchi_test1.antid_chi()
        self.assertTrue(jnp.all(jnp.isclose(guess_dchi.content, dchi_test1.content)))
        # chi integral
        self.assertTrue(jnp.all(jnp.isclose(guess_ichi.content, test1.content)))
        
        print('Testing chi derivative')
        print_fractional_error(guess_dchi.content, dchi_test1.content)
        print('Testing chi integral')
        print_fractional_error(guess_ichi.content, test1.content)

    def test_dphi_fft(self):
        '''
        phi derivatives
        '''
        nfp = np.random.randint(4)+2
        test_splines = rand_splines(10)

        test_diff = ChiPhiFunc_from_splines(splines=test_splines, nfp=nfp)
        ans = ChiPhiFunc_from_splines(splines=test_splines, nfp=nfp, dphi_order=1)

        print('Testing dphi based on FFT (tolerance:', diff_tolerance, ')')
        guess_fft = test_diff.dphi(mode=1)
        print_fractional_error(guess_fft.content, ans.content)
        self.assertTrue(jnp.all(is_roughly_close(guess_fft.content, ans.content)))
    
#     def test_dphi_pseudo_spectral(self):
#         '''
#         phi derivatives
#         '''
#         nfp = np.random.randint(4)+2
#         test_splines = rand_splines(10)

#         test_diff = ChiPhiFunc_from_splines(splines=test_splines, nfp=nfp)
#         ans = ChiPhiFunc_from_splines(splines=test_splines, nfp=nfp, dphi_order=1)
#         print('Testing dphi based on pseudo-spectral (tolerance:', diff_tolerance, ')')
#         guess_pseudo_spectral = test_diff.dphi(mode=2)
#         print_fractional_error(guess_pseudo_spectral.content, ans.content)
#         self.assertTrue(jnp.all(is_roughly_close(guess_pseudo_spectral.content, ans.content)))
    
    def test_phi_spectral_integral(self):
        ''' 
        phi integral 
        '''
        print('Testing spectral phi integral')
        amp=0.2
        nfp = np.random.randint(4)+2

        rands_i = np.random.randint(1,10, size=12)
        test_integral = amp*ChiPhiFunc(np.array([
                np.sin(rands_i[0]*points)+np.cos(rands_i[1]*points), 
                np.sin(rands_i[2]*points)+np.cos(rands_i[3]*points), 
                np.sin(rands_i[4]*points)+np.cos(rands_i[5]*points)
        ]),nfp, trig_mode = True)

        ans = amp*ChiPhiFunc(np.array([
                -1/rands_i[0]*np.cos(rands_i[0]*points) + 1/rands_i[1]*np.sin(rands_i[1]*points) +1/rands_i[0],
                -1/rands_i[2]*np.cos(rands_i[2]*points) + 1/rands_i[3]*np.sin(rands_i[3]*points) +1/rands_i[2],
                -1/rands_i[4]*np.cos(rands_i[4]*points) + 1/rands_i[5]*np.sin(rands_i[5]*points) +1/rands_i[4]
        ]), nfp, trig_mode = True)/nfp

        guess_fft = test_integral.integrate_phi_fft(zero_avg=False)
        print_fractional_error(guess_fft.content, ans.content)
        self.assertTrue(jnp.all(jnp.isclose(guess_fft.content, ans.content)))

    def test_diff_function(self):
        '''
        Testing the function diff() used for parsing
        '''
        # dphi mode
        nfp = np.random.randint(4)+2
        rands_i = np.random.randint(20,22, size=12)
        test_diff = ChiPhiFunc(np.array([
                np.cos(rands_i[0]*points) + np.sin(rands_i[1]*points) +1/rands_i[0]*np.cos(0),
                np.cos(rands_i[2]*points) + np.sin(rands_i[3]*points) +1/rands_i[2]*np.cos(0)
        ]), nfp, trig_mode = True)

        ans = ChiPhiFunc(np.array([
                -rands_i[0]*np.sin(rands_i[0]*points)+rands_i[1]*np.cos(rands_i[1]*points), 
                -rands_i[2]*np.sin(rands_i[2]*points)+rands_i[3]*np.cos(rands_i[3]*points)
        ]), nfp, trig_mode = True)*nfp

        ans2 = ChiPhiFunc(np.array([
                -rands_i[0]*rands_i[0]*np.cos(rands_i[0]*points)-rands_i[1]*rands_i[1]*np.sin(rands_i[1]*points), 
                -rands_i[2]*rands_i[2]*np.cos(rands_i[2]*points)-rands_i[3]*rands_i[3]*np.sin(rands_i[3]*points)
        ]), nfp, trig_mode = True)*nfp*nfp

        guess = diff(test_diff,False,1) # Dphi. The boolean is is_dchi
        guess2 = diff(test_diff,False,2) # Dphi. The boolean is is_dchi
        
        # Testing the function diff(). It is dphi() and dchi() in the backend
        # so the accuracy is not displayed again.
        self.assertTrue(jnp.all(is_roughly_close(guess.content, ans.content)))
        self.assertTrue(jnp.all(is_roughly_close(guess2.content, ans2.content)))

        # dchi mode
        rands_i = np.random.randint(1,5, size=12)
        a = np.sin(rands_i[0]*points)+np.cos(rands_i[1]*points) 
        b = np.sin(rands_i[2]*points)+np.cos(rands_i[3]*points)
        c = np.sin(rands_i[4]*points)+np.cos(rands_i[5]*points) 
        d = np.sin(rands_i[6]*points)+np.cos(rands_i[7]*points) 
        e = np.sin(rands_i[8]*points)+np.cos(rands_i[9]*points)
        test_diff = ChiPhiFunc(np.array([a,b,c,d,e]), nfp, trig_mode = True)

        ans = ChiPhiFunc(np.array([-4*e,-2*d,np.zeros_like(c),2*b,4*a]), nfp, trig_mode = True)
        # guess = test_diff.dchi(1)
        guess = diff_backend(test_diff,True,1) # Dphi. The boolean is is_dchi
        # guess = diff(test_diff,True,1) # Dphi. The boolean is is_dchi
        print('dchi')
        self.assertTrue(jnp.all(is_roughly_close(guess.content, ans.content)))
        
if __name__ == '__main__':
    unittest.main()


# (dphi + iota_coef[0] * dchi) Delta_n = inhomogenous
# This method evaluates the inhomogenous component, which serves as
# the input to solve_dphi_iota_dchi(iota, f).
# Uses B_denom_coef_cp[n], p_perp_coef_cp[n], iota[(n-1)/2 or n/2].
# Must be evaluated with Delta_n=0
from math import floor, ceil
from aqsc.math_utilities import *
# from jax import jit
# from functools import partial
# @partial(jit, static_argnums=(0,))
def eval_inhomogenous_Delta_n_cp(n,
    B_denom_coef_c,
    p_perp_coef_cp,
    Delta_coef_cp,
    iota_coef):
    def sum_arg_7(i257):
        # Child args for sum_arg_7
        def sum_arg_3(i224):
            # Child args for sum_arg_3
            return(Delta_coef_cp[i224]*diff(B_denom_coef_c[(-n)+2*i257-i224],True,1))

        def sum_arg_4(i254):
            # Child args for sum_arg_4
            return(B_denom_coef_c[i254]*diff(Delta_coef_cp[(-n)+2*i257-i254],True,1)*is_seq(n-i257,i257-i254))

        def sum_arg_6(i258):
            # Child args for sum_arg_6
            def sum_arg_5(i277):
                # Child args for sum_arg_5
                return(B_denom_coef_c[i258-i277]*B_denom_coef_c[i277])

            return(diff(p_perp_coef_cp[(-n)-i258+2*i257],True,1)*is_seq(n-i257,i257-i258)*py_sum(sum_arg_5,0,i258))

        return(2*is_seq(0,n-i257)*iota_coef[n-i257]*is_integer(n-i257)*py_sum(sum_arg_6,0,i257)+2*is_seq(0,n-i257)*iota_coef[n-i257]*is_integer(n-i257)*py_sum(sum_arg_4,0,i257)-is_seq(0,n-i257)*iota_coef[n-i257]*is_integer(n-i257)*is_seq(n-i257,i257)*py_sum(sum_arg_3,0,2*i257-n))

    def sum_arg_2(i256):
        # Child args for sum_arg_2
        def sum_arg_1(i220):
            # Child args for sum_arg_1
            return(B_denom_coef_c[i220]*B_denom_coef_c[n-i256-i220])

        return(diff(p_perp_coef_cp[i256],False,1)*py_sum(sum_arg_1,0,n-i256)+B_denom_coef_c[i256]*diff(Delta_coef_cp[n-i256],False,1))


    out = -(py_sum_parallel(sum_arg_7,ceil(n/2),floor(n))+2*is_seq(0,n)*is_integer(n)*py_sum_parallel(sum_arg_2,0,n))/(2*B_denom_coef_c[0])
    return(out)



# Evaluates p_perp_n
# Uses n, B_theta_coef_cp[n-2], B_psi_coef_cp[n-2],
# B_alpha_coef[first-order terms], B_denom_coef_c[n],
# p_perp_coef_cp[n-1], Delta_coef_cp[n-1],iota_coef[(n-3)/2 or (n-2)/2]
# Must be evaluated with p_perp_n=0
# The zeroth component contains B_psi_0:
# coeff = -2diff(Delta_coef_cp[0],'phi',1)/(n*B_alpha_coef[0]*B_denom_coef_c[0])
# coeff_dphi = -2(Delta_coef_cp[0]-1)/(n*B_alpha_coef[0]*B_denom_coef_c[0])
from math import floor, ceil
from aqsc.math_utilities import *
# from jax import jit
# from functools import partial
# @partial(jit, static_argnums=(0,))
def eval_p_perp_n_cp(n,
    B_theta_coef_cp,
    B_psi_coef_cp,
    B_alpha_coef,
    B_denom_coef_c,
    p_perp_coef_cp,
    Delta_coef_cp,
    iota_coef):
    def sum_arg_29(i294):
        # Child args for sum_arg_29
        def sum_arg_28(i256):
            # Child args for sum_arg_28
            return(B_psi_coef_cp[i256]*diff(Delta_coef_cp[n-i294-i256-2],False,1))

        return(B_denom_coef_c[i294]*py_sum(sum_arg_28,0,n-i294-2))

    def sum_arg_27(i290):
        # Child args for sum_arg_27
        def sum_arg_26(i288):
            # Child args for sum_arg_26
            return(Delta_coef_cp[i288]*diff(B_psi_coef_cp[n-i290-i288-2],False,1))

        return(B_denom_coef_c[i290]*py_sum(sum_arg_26,0,n-i290-2))

    def sum_arg_25(i285):
        # Child args for sum_arg_25
        def sum_arg_24(i286):
            # Child args for sum_arg_24
            def sum_arg_23(i258):
                # Child args for sum_arg_23
                return(B_psi_coef_cp[i258]*diff(Delta_coef_cp[(-n)-i286+2*i285-i258+2],True,1))

            return(B_denom_coef_c[i286]*is_seq(n-i285-2,i285-i286)*py_sum(sum_arg_23,0,(-n)-i286+2*i285+2))

        return(is_seq(0,n-i285-2)*iota_coef[n-i285-2]*is_integer(n-i285-2)*py_sum(sum_arg_24,0,i285))

    def sum_arg_22(i281):
        # Child args for sum_arg_22
        def sum_arg_21(i282):
            # Child args for sum_arg_21
            def sum_arg_20(i278):
                # Child args for sum_arg_20
                return(Delta_coef_cp[i278]*diff(B_psi_coef_cp[(-n)-i282+2*i281-i278+2],True,1))

            return(B_denom_coef_c[i282]*is_seq(n-i281-2,i281-i282)*py_sum(sum_arg_20,0,(-n)-i282+2*i281+2))

        return(is_seq(0,n-i281-2)*iota_coef[n-i281-2]*is_integer(n-i281-2)*py_sum(sum_arg_21,0,i281))

    def sum_arg_19(i271):
        # Child args for sum_arg_19
        def sum_arg_18(i272):
            # Child args for sum_arg_18
            return(B_denom_coef_c[i272]*Delta_coef_cp[(-n)-i272+2*i271]*is_seq(n-i271,i271-i272))

        return((is_seq(0,n-i271)*n-is_seq(0,n-i271)*i271)*B_alpha_coef[n-i271]*is_integer(n-i271)*py_sum(sum_arg_18,0,i271))

    def sum_arg_17(i265):
        # Child args for sum_arg_17
        def sum_arg_16(i266):
            # Child args for sum_arg_16
            return(B_denom_coef_c[i266]*diff(B_psi_coef_cp[(-n)-i266+2*i265+2],True,1)*is_seq(n-i265-2,i265-i266))

        return(is_seq(0,n-i265-2)*iota_coef[n-i265-2]*is_integer(n-i265-2)*py_sum(sum_arg_16,0,i265))

    def sum_arg_15(i264):
        # Child args for sum_arg_15
        return(B_denom_coef_c[i264]*diff(B_psi_coef_cp[n-i264-2],False,1))

    def sum_arg_14(i259):
        # Child args for sum_arg_14
        def sum_arg_13(i260):
            # Child args for sum_arg_13
            return(i260*B_denom_coef_c[i260]*Delta_coef_cp[(-n)-i260+2*i259]*is_seq(n-i259,i259-i260))

        return(is_seq(0,n-i259)*B_alpha_coef[n-i259]*is_integer(n-i259)*py_sum(sum_arg_13,0,i259))

    def sum_arg_12(i251):
        # Child args for sum_arg_12
        def sum_arg_11(i252):
            # Child args for sum_arg_11
            def sum_arg_10(i246):
                # Child args for sum_arg_10
                return(B_denom_coef_c[i246]*B_denom_coef_c[(-n)-i252+2*i251-i246])

            return(i252*p_perp_coef_cp[i252]*is_seq(n-i251,i251-i252)*py_sum(sum_arg_10,0,(-n)-i252+2*i251))

        return(is_seq(0,n-i251)*B_alpha_coef[n-i251]*is_integer(n-i251)*py_sum(sum_arg_11,0,i251))

    def sum_arg_9(i291):
        # Child args for sum_arg_9
        def sum_arg_7(i292):
            # Child args for sum_arg_7
            return(is_seq(0,(-n)-i292+2*i291)*B_denom_coef_c[i292]*B_theta_coef_cp[(-n)-i292+2*i291]*is_integer((-n)-i292+2*i291)*is_seq((-n)-i292+2*i291,i291-i292))

        def sum_arg_8(i292):
            # Child args for sum_arg_8
            return(is_seq(0,(-n)-i292+2*i291)*B_denom_coef_c[i292]*B_theta_coef_cp[(-n)-i292+2*i291]*is_integer((-n)-i292+2*i291)*is_seq((-n)-i292+2*i291,i291-i292))

        return(iota_coef[n-i291]*(n*py_sum(sum_arg_8,0,i291)-i291*py_sum(sum_arg_7,0,i291)))

    def sum_arg_6(i275):
        # Child args for sum_arg_6
        def sum_arg_3(i276):
            # Child args for sum_arg_3
            def sum_arg_2(i274):
                # Child args for sum_arg_2
                return(is_seq(0,(-n)-i276+2*i275-i274)*Delta_coef_cp[i274]*B_theta_coef_cp[(-n)-i276+2*i275-i274]*is_integer((-n)-i276+2*i275-i274)*is_seq((-n)-i276+2*i275-i274,(-i276)+i275-i274))

            return(B_denom_coef_c[i276]*py_sum(sum_arg_2,0,i275-i276))

        def sum_arg_5(i276):
            # Child args for sum_arg_5
            def sum_arg_4(i274):
                # Child args for sum_arg_4
                return(is_seq(0,(-n)-i276+2*i275-i274)*Delta_coef_cp[i274]*B_theta_coef_cp[(-n)-i276+2*i275-i274]*is_integer((-n)-i276+2*i275-i274)*is_seq((-n)-i276+2*i275-i274,(-i276)+i275-i274))

            return(B_denom_coef_c[i276]*py_sum(sum_arg_4,0,i275-i276))

        return(iota_coef[n-i275]*(n*py_sum(sum_arg_5,0,i275)-i275*py_sum(sum_arg_3,0,i275)))

    def sum_arg_1(i267):
        # Child args for sum_arg_1
        return((is_seq(0,n-i267)*B_denom_coef_c[2*i267-n]*n-is_seq(0,n-i267)*i267*B_denom_coef_c[2*i267-n])*B_alpha_coef[n-i267]*is_integer(n-i267)*is_seq(n-i267,i267))


    out = -((-4*py_sum_parallel(sum_arg_9,ceil(n/2),floor(n)))+4*py_sum_parallel(sum_arg_6,ceil(n/2),floor(n))+4*is_seq(0,n-2)*is_integer(n-2)*py_sum_parallel(sum_arg_29,0,n-2)+4*is_seq(0,n-2)*is_integer(n-2)*py_sum_parallel(sum_arg_27,0,n-2)+4*py_sum_parallel(sum_arg_25,ceil(n/2)-1,floor(n)-2)+4*py_sum_parallel(sum_arg_22,ceil(n/2)-1,floor(n)-2)-4*py_sum_parallel(sum_arg_19,ceil(n/2),floor(n))-4*py_sum_parallel(sum_arg_17,ceil(n/2)-1,floor(n)-2)-4*is_seq(0,n-2)*is_integer(n-2)*py_sum_parallel(sum_arg_15,0,n-2)-py_sum_parallel(sum_arg_14,ceil(n/2),floor(n))+2*py_sum_parallel(sum_arg_12,ceil(n/2),floor(n))+4*py_sum_parallel(sum_arg_1,ceil(n/2),floor(n)))/(2*B_alpha_coef[0]*B_denom_coef_c[0]**2*n)
    return(out)
